/*
 * Copyright 2010 James Pether Sörling
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: WorldbankApiImpl.java 4527 2010-09-17 19:13:17Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/service.external.worldbank-1.0.0-beta-1/src/main/java/com/hack23/cia/service/external/worldbank/impl/WorldbankApiImpl.java $
*/
package com.hack23.cia.service.external.worldbank.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.oxm.Unmarshaller;
import org.springframework.stereotype.Component;

import com.hack23.cia.model.external.worldbank.countries.impl.CountriesElement;
import com.hack23.cia.model.external.worldbank.countries.impl.CountryElement;
import com.hack23.cia.model.external.worldbank.data.impl.Data;
import com.hack23.cia.model.external.worldbank.data.impl.DataElement;
import com.hack23.cia.model.external.worldbank.indicators.impl.IndicatorElement;
import com.hack23.cia.model.external.worldbank.indicators.impl.IndicatorsElement;
import com.hack23.cia.service.external.common.impl.AbstractXmlAgentImpl;
import com.hack23.cia.service.external.worldbank.api.WorldBankApi;

/**
 * The Class WorldbankApiImpl.
 */
@Component
public class WorldbankApiImpl extends AbstractXmlAgentImpl implements WorldBankApi {

	/** The COUNTRIES. */
	private static String COUNTRIES= "http://open.worldbank.org/countries?per_page=300";

	/** The INDICATO r_ countr y_ data. */
	private static String INDICATOR_COUNTRY_DATA ="http://open.worldbank.org/countries/${COUNTRY_KEY}/indicators[${INDICATOR_KEY}]?per_page=1000";

	/** The INDICATORS. */
	private static String INDICATORS= "http://open.worldbank.org/indicators?per_page=3000";

	/** The countries unmarshaller. */
	@Autowired
	@Qualifier("worldbankOrgCountriesMarshaller")
	private Unmarshaller countriesUnmarshaller;

	/** The data unmarshaller. */
	@Autowired
	@Qualifier("worldbankOrgDataMarshaller")
	private Unmarshaller dataUnmarshaller;

	/** The indicators unmarshaller. */
	@Autowired
	@Qualifier("worldbankOrgIndicatorsMarshaller")
	private Unmarshaller indicatorsUnmarshaller;


	/* (non-Javadoc)
	 * @see com.hack23.cia.service.component.agent.impl.worldbank.api.WorlbankApi#getCountries()
	 */
	@Override
	public List<CountryElement> getCountries() throws Exception {
		return ((CountriesElement) this.unmarshallXml(countriesUnmarshaller, COUNTRIES,null,"xmlns:wb=\"http://www.worldbank.org\"", "xmlns:wb=\"http://countries.worldbank.external.model.cia.hack23.com/impl\"")).getCountry();	
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.component.agent.impl.worldbank.api.WorlbankApi#getData(java.lang.String, java.lang.String)
	 */
	@Override
	public List<Data> getData(final String countryCode,final String indicatorId) throws Exception {
		String url = INDICATOR_COUNTRY_DATA.replace("${COUNTRY_KEY}",countryCode);
		url = url.replace("${INDICATOR_KEY}", indicatorId);
		return ((DataElement) this.unmarshallXml(dataUnmarshaller, url,null,"xmlns:wb=\"http://www.worldbank.org\"", "xmlns:wb=\"http://data.worldbank.external.model.cia.hack23.com/impl\"")).getData();
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.component.agent.impl.worldbank.api.WorlbankApi#getIndicators()
	 */
	@Override
	public List<IndicatorElement> getIndicators() throws Exception {
		return ((IndicatorsElement) this.unmarshallXml(indicatorsUnmarshaller, INDICATORS,null,"xmlns:wb=\"http://www.worldbank.org\"", "xmlns:wb=\"http://indicators.worldbank.external.model.cia.hack23.com/impl\"")).getIndicator();
	}

}
